@@ -6,46 +6,53 @@ from .utils import toeplitz_block, get_value, fsolve, extend
 from autograd.extend import primitive, defvjp, vspace
 from autograd import grad, vector_jacobian_product
 import autograd.numpy as npa
-
 """ Define here various primitives needed for the main code 
 To use with both numpy and autograd backends, define the autograd primitive of 
 a numpy function fnc as fnc_ag, and then define the vjp"""
 
-def T(x): return np.swapaxes(x, -1, -2)
+
+def T(x):
+    return np.swapaxes(x, -1, -2)
+
 
 """=========== EXPAND ARRAY TO A GIVEN SHAPE =========== """
 
-# extend(vals, inds, shape) makes an array of shape `shape` where indices 
-# `inds` have values `vals` 
+# extend(vals, inds, shape) makes an array of shape `shape` where indices
+# `inds` have values `vals`
 extend_ag = primitive(extend)
 
+
 def vjp_maker_extend(ans, vals, inds, shape):
     def vjp(g):
         return g[inds]
+
     return vjp
 
-defvjp(extend_ag, vjp_maker_extend, None, None)
 
+defvjp(extend_ag, vjp_maker_extend, None, None)
 """=========== NP.SQRT STABLE AROUND 0 =========== """
 sqrt_ag = primitive(np.sqrt)
 
+
 def vjp_maker_sqrt(ans, x):
     def vjp(g):
-        return g * 0.5 * (x + 1e-10)**0.5/(x + 1e-10)
+        return g * 0.5 * (x + 1e-10)**0.5 / (x + 1e-10)
         # return np.where(np.abs(x) > 1e-10, g * 0.5 * x**-0.5, 0.)
+
     return vjp
 
-defvjp(sqrt_ag, vjp_maker_sqrt)
 
+defvjp(sqrt_ag, vjp_maker_sqrt)
 """=========== TOEPLITZ-BLOCK =========== """
 
 toeplitz_block_ag = primitive(toeplitz_block)
 
+
 def vjp_maker_TB_T1(Tmat, n, T1, T2):
     """ Gives vjp for Tmat = toeplitz_block(n, T1, T2) w.r.t. T1"""
     def vjp(v):
         ntot = Tmat.shape[0]
-        p = int(ntot/n) # Linear size of each block
+        p = int(ntot / n)  # Linear size of each block
         vjac = np.zeros(T1.shape, dtype=np.complex128)
 
         for ind1 in range(n):
@@ -61,11 +68,12 @@ def vjp_maker_TB_T1(Tmat, n, T1, T2):
 
     return vjp
 
+
 def vjp_maker_TB_T2(Tmat, n, T1, T2):
     """ Gives vjp for Tmat = toeplitz_block(n, T1, T2) w.r.t. T2"""
     def vjp(v):
         ntot = Tmat.shape[0]
-        p = int(ntot/n) # Linear size of each block
+        p = int(ntot / n)  # Linear size of each block
         vjac = np.zeros(T2.shape, dtype=np.complex128)
 
         for ind1 in range(n):
@@ -81,27 +89,28 @@ def vjp_maker_TB_T2(Tmat, n, T1, T2):
 
     return vjp
 
-defvjp(toeplitz_block_ag, None, vjp_maker_TB_T1, vjp_maker_TB_T2)
 
+defvjp(toeplitz_block_ag, None, vjp_maker_TB_T1, vjp_maker_TB_T2)
 """=========== NUMPY.LINALG.EIGH =========== """
 
 eigh_ag = primitive(np.linalg.eigh)
 
+
 def vjp_maker_eigh(ans, x, UPLO='L'):
     """Gradient for eigenvalues and vectors of a hermitian matrix."""
     N = x.shape[-1]
-    w, v = ans              # Eigenvalues, eigenvectors.
+    w, v = ans  # Eigenvalues, eigenvectors.
     vc = np.conj(v)
-    
+
     def vjp(g):
-        wg, vg = g          # Gradient w.r.t. eigenvalues, eigenvectors.
+        wg, vg = g  # Gradient w.r.t. eigenvalues, eigenvectors.
         w_repeated = np.repeat(w[:, np.newaxis], N, axis=-1)
 
         # Eigenvalue part
-        vjp_temp = np.dot(vc * wg[np.newaxis, :], T(v)) 
+        vjp_temp = np.dot(vc * wg[np.newaxis, :], T(v))
 
         # Add eigenvector part only if non-zero backward signal is present.
-        # This can avoid NaN results for degenerate cases if the function 
+        # This can avoid NaN results for degenerate cases if the function
         # depends on the eigenvalues only.
         if np.any(vg):
             off_diag = np.ones((N, N)) - np.eye(N)
@@ -117,14 +126,14 @@ def vjp_maker_eigh(ans, x, UPLO='L'):
             tri = np.tile(np.tril(np.ones(N), -1), reps)
         elif UPLO == 'U':
             tri = np.tile(np.triu(np.ones(N), 1), reps)
-        
+
         return np.real(vjp_temp)*np.eye(vjp_temp.shape[-1]) + \
             (vjp_temp + np.conj(T(vjp_temp))) * tri
 
     return vjp
 
-defvjp(eigh_ag, vjp_maker_eigh)
 
+defvjp(eigh_ag, vjp_maker_eigh)
 """=========== MATRIX INVERSE =========== """
 """We define this here without the `einsum` notation that's used in autograd.
 `einsum` allows broadcasting (which we don't care about), but is slower 
@@ -133,10 +142,12 @@ defvjp(eigh_ag, vjp_maker_eigh)
 
 inv_ag = primitive(np.linalg.inv)
 
+
 def vjp_maker_inv(ans, x):
     return lambda g: -np.dot(np.dot(T(ans), g), T(ans))
-defvjp(inv_ag, vjp_maker_inv)
 
+
+defvjp(inv_ag, vjp_maker_inv)
 """=========== SCIPY.SPARSE.LINALG.EIGSH =========== """
 
 eigsh_ag = primitive(sp.linalg.eigsh)
@@ -147,16 +158,16 @@ eigsh_ag = primitive(sp.linalg.eigsh)
 #     N = x.shape[-1]
 #     w, v = ans              # Eigenvalues, eigenvectors.
 #     vc = np.conj(v)
-    
+
 #     def vjp(g):
 #         wg, vg = g          # Gradient w.r.t. eigenvalues, eigenvectors.
 #         w_repeated = np.repeat(w[..., np.newaxis], numeig, axis=-1)
 
 #         # Eigenvalue part
-#         vjp_temp = np.dot(vc * wg[..., np.newaxis, :], T(v)) 
+#         vjp_temp = np.dot(vc * wg[..., np.newaxis, :], T(v))
 
 #         # Add eigenvector part only if non-zero backward signal is present.
-#         # This can avoid NaN results for degenerate cases if the function 
+#         # This can avoid NaN results for degenerate cases if the function
 #         # depends on the eigenvalues only.
 #         if np.any(vg):
 #             off_diag = np.ones((numeig, numeig)) - np.eye(numeig)
@@ -167,13 +178,14 @@ eigsh_ag = primitive(sp.linalg.eigsh)
 
 #     return vjp
 
+
 def vjp_maker_eigsh(ans, mat, **kwargs):
     """Steven Johnson method extended to a Hermitian matrix
     https://math.mit.edu/~stevenj/18.336/adjoint.pdf
     """
     numeig = kwargs['k']
     N = mat.shape[0]
-    
+
     def vjp(g):
         vjp_temp = np.zeros_like(mat)
         for iv in range(numeig):
@@ -184,33 +196,33 @@ def vjp_maker_eigsh(ans, mat, **kwargs):
             vg = g[1][:, iv]
 
             # Eigenvalue part
-            vjp_temp += ag*np.outer(vc, v)
+            vjp_temp += ag * np.outer(vc, v)
 
             # Add eigenvector part only if non-zero backward signal is present.
-            # This can avoid NaN results for degenerate cases if the function 
+            # This can avoid NaN results for degenerate cases if the function
             # depends on the eigenvalues only.
             if np.any(vg):
                 # Projection operator on space orthogonal to v
                 P = np.eye(N, N) - np.outer(vc, v)
-                Amat = T(mat - a*np.eye(N, N))
+                Amat = T(mat - a * np.eye(N, N))
                 b = P.dot(vg)
 
                 # Initial guess orthogonal to v
                 v0 = P.dot(np.random.randn(N))
 
-                # Find a solution lambda_0 using conjugate gradient 
+                # Find a solution lambda_0 using conjugate gradient
                 (l0, _) = sp.linalg.cg(Amat, b, x0=v0, atol=0)
                 # Project to correct for round-off errors
                 l0 = P.dot(l0)
 
                 vjp_temp -= np.outer(l0, v)
- 
+
         return vjp_temp
 
     return vjp
 
-defvjp(eigsh_ag, vjp_maker_eigsh)
 
+defvjp(eigsh_ag, vjp_maker_eigsh)
 """=========== NUMPY.INTERP =========== """
 """This implementation might not be covering the full scope of the numpy.interp
 function, but it covers everything we need
@@ -218,22 +230,24 @@ function, but it covers everything we need
 
 interp_ag = primitive(np.interp)
 
+
 def vjp_maker_interp(ans, x, xp, yp):
     """Construct the vjp of interp(x, xp, yp) w.r.t. yp
     """
-
     def vjp(g):
         dydyp = np.zeros((x.size, xp.size))
         for ix in range(x.size):
             indx = np.searchsorted(xp, x[ix]) - 1
-            dydyp[ix, indx] = 1 - (x[ix] - xp[indx])/(xp[indx+1] - xp[indx])
-            dydyp[ix, indx+1] = (x[ix] - xp[indx])/(xp[indx+1] - xp[indx])
+            dydyp[ix,
+                  indx] = 1 - (x[ix] - xp[indx]) / (xp[indx + 1] - xp[indx])
+            dydyp[ix,
+                  indx + 1] = (x[ix] - xp[indx]) / (xp[indx + 1] - xp[indx])
         return np.dot(g, dydyp)
+
     return vjp
 
+
 defvjp(interp_ag, None, None, vjp_maker_interp)
-
-
 """=========== SOLVE OF f(x, y) = 0 W.R.T. X =========== """
 fsolve_ag = primitive(fsolve)
 """fsolve_ag(fun, lb, ub, *args) solves fun(x, *args) = 0 for lb <= x <= ub
@@ -241,6 +255,7 @@ fsolve_ag = primitive(fsolve)
     args can be anything
 """
 
+
 def vjp_factory_fsolve(ginds):
     """
     Factory function defining the vjp_makers for a generic fsolve_ag with 
@@ -255,36 +270,34 @@ def vjp_factory_fsolve(ginds):
     """
 
     # Gradients w.r.t fun, lb and ub are not computed
-    vjp_makers = [None, None, None] 
+    vjp_makers = [None, None, None]
 
     def vjp_single_arg(ia):
-
         def vjp_maker(ans, *args):
             f = args[0]
             fargs = args[3:]
             dfdx = grad(f, 0)(ans, *fargs)
-            dfdy = grad(f, ia+1)(ans, *fargs)
+            dfdy = grad(f, ia + 1)(ans, *fargs)
 
-            def vjp(g):       
-                return np.dot(g, -1/dfdx * dfdy)
+            def vjp(g):
+                return np.dot(g, -1 / dfdx * dfdy)
 
             return vjp
 
         return vjp_maker
 
     for (ia, gind) in enumerate(ginds):
-        if gind==True:
+        if gind == True:
             vjp_makers.append(vjp_single_arg(ia=ia))
         else:
             vjp_makers.append(None)
 
     return tuple(vjp_makers)
 
+
 # NB: This definition is for the specific fsolve with three arguments
 # used for the guided modes!!!
 defvjp(fsolve_ag, *vjp_factory_fsolve([False, True, True]))
-
-
 """=========== MAP FUNCTION EVALUATION =========== """
 """ A variation of the `functools.map` function applied to a list of functions,
     defined as follows
@@ -295,6 +308,7 @@ defvjp(fsolve_ag, *vjp_factory_fsolve([False, True, True]))
     array of the same size as `fns`.
 """
 
+
 @primitive
 def fmap(fns, params):
     """ autograd-ready version of functools.fmap applied to a list of functions
@@ -316,4 +330,5 @@ def vjp_maker_fmap(ans, fns, params):
     # this literally does the vector-jacobian product
     return lambda v: np.dot(v.T, grads)
 
+
 defvjp(fmap, None, vjp_maker_fmap)
